@@ -62,7 +62,7 @@ func testGetLineCount(t *testing.T, reader *ReaderImpl) {
 	}
 }
 
-func firstLine(inputLines *InputLines) linemetadata.Index {
+func firstLine(inputLines InputLines) linemetadata.Index {
 	return inputLines.Lines[0].Index
 }
 
@@ -316,7 +316,8 @@ func TestReadTextDone(t *testing.T) {
 
 // JSON should be auto detected and formatted
 func TestFormatJson(t *testing.T) {
-	jsonStream := strings.NewReader(`{"key": "value"}`)
+	// Note the space after "key" to verify formatting actually happens
+	jsonStream := strings.NewReader(`{"key" :"value"}`)
 	testMe, err := NewFromStream(
 		"JSON test",
 		jsonStream,
@@ -337,7 +338,8 @@ func TestFormatJson(t *testing.T) {
 }
 
 func TestFormatJsonArray(t *testing.T) {
-	jsonStream := strings.NewReader(`[{"key": "value"}]`)
+	// Note the space after "key" to verify formatting actually happens
+	jsonStream := strings.NewReader(`[{"key" :"value"}]`)
 	testMe, err := NewFromStream(
 		"JSON test",
 		jsonStream,
