@@ -348,8 +348,9 @@ func (p *Pager) findFirstHit(startPosition linemetadata.Index, beforePosition *l
 // help large file search performance.
 func _findFirstHit(reader reader.Reader, startPosition linemetadata.Index, pattern regexp.Regexp, beforePosition *linemetadata.Index, backwards bool) *linemetadata.Index {
 	searchPosition := startPosition
+	lineCache := searchLineCache{}
 	for {
-		line := reader.GetLine(searchPosition)
+		line := lineCache.getLine(reader, searchPosition, backwards)
 		if line == nil {
 			// No match, give up
 			return nil
@@ -568,15 +569,16 @@ func (p *Pager) scrollRightToSearchHits() bool {
 		// Minus one to account for the scroll-left marker that will cover the
 		// first column after scrolling.
 		scrollToColumn := firstNotVisibleColumn - 1
-		if scrollToColumn > maxLeftmostColumn {
-			scrollToColumn = maxLeftmostColumn
-		}
 
 		p.showLineNumbers = false
 		p.leftColumnZeroBased = scrollToColumn
 
 		if p.searchHitIsVisible() {
-			// Found it!
+			// A new hit showed up!
+			if p.leftColumnZeroBased > maxLeftmostColumn {
+				// Scrolled beyond max, adjust
+				p.leftColumnZeroBased = maxLeftmostColumn
+			}
 			return true
 		}
 	}
